import "CubePlugins.MinstrelBuffII.Themes.ThemeList";
import "CubePlugins.MinstrelBuffII.Themes.WarsteedThemeList";
import "CubePlugins.MinstrelBuffII.SettingsEncoder"

Settings = class();

PluginName = plugin:GetName();
PluginVersion = plugin:GetVersion();

function Settings:Constructor()
	self:Load();
	self:DetectLocalization();
end

function Settings:FixupNumber(key)
	self.settingsTable[key] = euroNormalize(self.settingsTable[key]);
end

function Settings:SetDefaultIfMissing(key, value)
	if (self.settingsTable[key] == nil) then
		self.settingsTable[key] = value;
	end
end

function Settings:Load()
	local wasSettingsFileCorrect, temp = pcall(function() self.settingsTable = PatchDataLoad(Turbine.DataScope.Character, "MinstrelBuff"); end);

	if (self.settingsTable == nil) then
		self.settingsTable = {};
	end

	self:SetDefaultIfMissing("MainPosX", 100);
	self:SetDefaultIfMissing("MainPosY", 100);
	self:SetDefaultIfMissing("SoliloquyPosX", 200);
	self:SetDefaultIfMissing("SoliloquyPosY", 200);
	self:SetDefaultIfMissing("MelodicInterludePosX", 300);
	self:SetDefaultIfMissing("MelodicInterludePosY", 300);
	-- MelodicInterludeWidth is stored as a percentage of the screen width.
	self:SetDefaultIfMissing("MelodicInterludeWidth", MelodicInterludeWindow.GetDefaultWidth() / Turbine.UI.Display:GetWidth());
	-- MainWindowWidth is also stored as a percentage of the screen width.
	self:SetDefaultIfMissing("MainWindowWidth", BuffWindow.GetDefaultWidth() / Turbine.UI.Display:GetWidth());
	self:SetDefaultIfMissing("ThemeIndex", 1);
	self:SetDefaultIfMissing("EffectWindowOnlyVisibleInCombat", false);
	self:SetDefaultIfMissing("SolilquyWindowUsed", false);

	self:FixupNumber("ThemeIndex");
	self:FixupNumber("MainPosX");
	self:FixupNumber("MainPosY");
	self:FixupNumber("SoliloquyPosX");
	self:FixupNumber("SoliloquyPosY");
	self:FixupNumber("MelodicInterludePosX");
	self:FixupNumber("MelodicInterludePosY");
	self:FixupNumber("MelodicInterludeWidth");
	self:FixupNumber("MainWindowWidth");

	-- events
	self.SettingChanged = {};

	-- events to be removed:
	self.ThemeChanged = nil;
	if (not wasSettingsFileCorrect) then
		self:Save();
		Turbine.Shell.WriteLine("Minstrel Buff - Settings file corrupt. Created a new file");
	end
	--Turbine.Shell.WriteLine( "Settings:Load" );
end

function Settings:Save()
	if (self.settingsTable ~= nil) then
		PatchDataSave(Turbine.DataScope.Character, "MinstrelBuff", self.settingsTable);
	end
end

-- Generic Window Position Functions:
-- Pass in names like "Main", "Soliloquy"

function Settings:GetWindowPosition(windowName)
	local x = 200;
	local y = 200;

	if (self.settingsTable[windowName .. "PosX"] ~= nil) then
		x = self.settingsTable[windowName .. "PosX"];
	end
	if (self.settingsTable[windowName .. "PosY"] ~= nil) then
		y = self.settingsTable[windowName .. "PosY"];
	end
	return x, y;
end

function Settings:SetWindowPosition(windowName, posx, posy)
	self.settingsTable[windowName .. "PosX"] = posx;
	self.settingsTable[windowName .. "PosY"] = posy;
	self:Save();
end

-- End Generic Window Position Functions

-- Generic Helper Functions:

function Settings:GetSetting(settingName)
	return self.settingsTable[settingName];
end

function Settings:SetSetting(settingName, settingValue)
	self.settingsTable[settingName] = settingValue;
	self:Save();
	self:TriggerCallback(settingName);
end

function Settings:AddCallback(settingName, callback)
	self.SettingChanged[settingName] = callback;
end

function Settings:TriggerCallback(settingName)
	local callback = self.SettingChanged[settingName];
	if (callback ~= nil and type(callback) == "function") then
		callback();
	end
end

-- End Generic Helper Functions

function Settings:DetectLocalization()
	if Turbine.Shell.IsCommand("hilfe") then
        self.Localization = "de";
    elseif Turbine.Shell.IsCommand("aide") then
        self.Localization = "fr";
    else
		self.Localization = "en";
	end
end

function Settings:GetEffectLocalizationNames()
	return self.Localization;
end

-- Theme Settings
function Settings:DoesThemeSupportExtraAnthems()
	local theme = self:GetTheme();
	return theme.supportsExtraAnthems;
end

function Settings:GetWindowBackground(anthemCount)
	local theme = self:GetTheme();
	local supports = self:DoesThemeSupportExtraAnthems();
	if (not supports) then
		return theme.windowBackground;
	end
	if (anthemCount == nil) then anthemCount = 3; end

	local countToUse = anthemCount;
	if (anthemCount <= 3) then countToUse = 3;
	elseif (anthemCount > 6) then countToUse = 6;
	end

	return theme.windowBackgrounds[countToUse];
end

function Settings:GetFixedAnthemPositions()
	local theme = self:GetTheme();
	return theme.fixedAnthems;
end

function Settings:GetShowAnthems()
	local theme = self:GetTheme();
	return theme.showAnthems;
end

function Settings:GetShowAnthemTime()
	local theme = self:GetTheme();
	if (theme.showAnthemTime ~= nil) then
		return theme.showAnthemTime;
	else
		return true;
	end
end

function Settings:GetThemeList()
	return CubePlugins.MinstrelBuffII.Themes.ThemeList;
end

function Settings:GetTheme()
	if (self.isOnWarsteed) then
		return CubePlugins.MinstrelBuffII.Themes.WarsteedThemeList[self:GetWarsteedThemeIndex()];
	else
		return CubePlugins.MinstrelBuffII.Themes.ThemeList[self:GetThemeIndex()];
	end
end

function Settings:GetThemeIndex()
	return self.settingsTable["ThemeIndex"];
end

function Settings:GetWarsteedThemeIndex()
	return 1;
end

function Settings:SetThemeIndex(value)
	if (self.settingsTable["ThemeIndex"] == value) then return; end

	self.settingsTable["ThemeIndex"] = value;
	self:Save();
	if (type(self.ThemeChanged) == "function") then
		self.ThemeChanged();
	end
end

function Settings:SetIsOnWarsteed(value)
	if (self.isOnWarsteed ~= value) then
		self.isOnWarsteed = value;
		if (type(self.ThemeChanged) == "function") then
			self.ThemeChanged();
		end
	end
end
